// ScreenSizer Module
// Jeffrey J Weimer
// SVN History: $Revision: 839 $ on $Date: 2008-07-10 10:24:31 -0500 (Thu, 10 Jul 2008) $
//
// See the Help file for further information

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.7

#pragma IndependentModule=ScreenSizer

// Initialize
//   call to the function as ScreenSizer#Initialize(winXX)
//	winXX: 0 - do not change frame size on WinXX systems (default!)
//	winXX: 1 - use full frame size on WinXX systems

Function Initialize(winXX)
	variable winXX

	if ((winXX!=0) && (winXX!=1))
		winXX = 0
	endif
	
	if (DataFolderExists("root:Packages:ScreenSizer")==0)
	
		// initalize from clean start
	
		string cdf = GetDataFolder(1)
		
		NewDataFolder/O/S root:Packages
		NewDataFolder/O/S ScreenSizer
		variable/G scrwpxl, scrhpxl, scrwpnt, scrhpnt
		variable/G scrleft = 0, scrtop = 0, scrright = 100, scrbottom = 100
		SetDataFolder $cdf

	endif

	NVAR scrwx = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhx = root:Packages:ScreenSizer:scrhpxl
	NVAR scrwn = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhn = root:Packages:ScreenSizer:scrhpnt

	if (cmpstr(IgorInfo(2),"Macintosh")==0)

		string IgorStuff=""
		variable scr0,scr1,scr2
	
		IgorStuff=IgorInfo(0)
		scr0 = strsearch(IgorStuff,"RECT",0)
		scr1 = strsearch(IgorStuff,",",scr0+9)
		scr2 = strlen(IgorStuff)-2
		
		// values are in RELATIVE PIXELS (for tables and graphs)!!!!
		
		scrwx = str2num(IgorStuff[scr0+9,scr1-1])/100
		scrhx = str2num(IgorStuff[scr1+1,scr2])/100
	else

		// maximize the frame
		
		if (winXX)
			MoveWindow/F 2, 2, 2, 2
		endif
		GetWindow kwFrameInner wsizeDC

		// values are in RELATIVE PIXELS (for panels)!!!!
		
		scrwx = (v_right-v_left)/100
		scrhx = (v_bottom-v_top)/100
	endif
	
	// values in RELATIVE POINTS (for tables and graphs)
	
	scrwn = scrwx*72/ScreenResolution
	scrhn = scrhx*72/ScreenResolution
	
	return 0
End

// Set Frame

Function SetFrame(left, top, right, bottom)
	variable left, top, right, bottom
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom
	
	scrleft = left
	scrtop = top
	scrright = right
	scrbottom = bottom
	return 0
end

// Move Front Window

Function MoveFrontWindow(left, top, right, bottom)
	variable left, top, right, bottom
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	MoveWindow left,top,right,bottom
	return 0
end

// Move Command Window

Function MoveCmdWindow(left, top, right, bottom)
	variable left, top, right, bottom
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	left=(scrwpnt*(scrleft + (scrright - scrleft)*left/100))
	top=(scrhpnt*(scrtop + (scrbottom-scrtop)*top/100))
	right=(scrwpnt*(scrleft + (scrright - scrleft)*right/100))
	bottom=(scrhpnt*(scrtop + (scrbottom-scrtop)*bottom/100))
	
	MoveWindow/C left,top,right,bottom
	return 0
end

// Make a Panel

Function MakePanel(left, top, width, height, strname, [other])
	variable left, top, width, height
	string strname, other
	
	variable right, bottom
	string exeCmd
	
	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrright = root:Packages:ScreenSizer:scrright
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl

	if (left>=0)
		left=(scrwpxl*(scrleft + (scrright - scrleft)*left/100))
	else
		left=(scrwpxl*(scrright + (scrright - scrleft)*left/100)) - width
	endif
	if (top>=0)
		top=(scrhpxl*(scrtop + (scrbottom-scrtop)*top/100))
	else
		top=(scrhpxl*(scrbottom + (scrbottom-scrtop)*top/100)) - height
	endif
	right=left+width
	bottom=top+height
	
	if (ParamIsDefault(other))
		sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)as \"%s\"", left, top, right, bottom, strname
	else
		sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)%s as \"%s\"", left, top, right, bottom, other, strname
	endif
	execute/Q exeCmd
	return 0
	
end

// Make an External Panel

Function MakeExtPanel(side, offset, width, height, strname, [other])
	variable side, offset, width, height
	string strname, other
	
	variable left, right, top, bottom
	string exeCmd
	
	Initialize(0)

	switch(side)
		case 0:		
		case 1:
			left = offset
			top = 0
			right = width+offset
			bottom = height
			break
		default:
			left = 0
			top = offset
			right = width
			bottom = height+offset
			break
	endswitch
	
	if (ParamIsDefault(other))
		other = "/HOST=#"
	endif
	other += "/EXT=" + num2str(side)
	sprintf exeCmd "NewPanel/W=(%g,%g,%g,%g)%s as \"%s\"", left, top, right, bottom, other, strname
	execute/Q exeCmd
	return 0
	
end

// Save Window Coordinates

Function SaveWinCoords([name])
	string name
	
	string cdf = GetDataFolder(1), theCoords=""

	Initialize(0)
	
	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	if (ParamIsDefault(name))
		name = WinName(0,(1+2+4+16+64),1)
	endif
	switch(WinType(name))
		case 0:
			return -1
			break
		case 13:
			return -1
			break
		default:
			GetWindow $name wsize
			sprintf theCoords, "left=%g;top=%g;right=%g;bottom=%g;width=%g;height=%g", V_left, V_top, V_right, V_bottom, scrwpnt, scrhpnt
			SetWindow $name userData(ScreenSizer)=theCoords
			break
	endswitch
	return 0

end

// Restore Window Coordinates
// how:
//	-1 - restore LEFT, TOP and WIDTH, HEIGHT according to FIRST SCREEN
//	0 (default) - move LEFT, TOP according to NEW SCREEN and keep WIDTH, HEIGHT according to FIRST SCREEN
//	1 - move LEFT, TOP and WIDTH, HEIGHT according to NEW SCREEN
//	2 - move LEFT, TOP and WIDTH, HEIGHT according to NEW SCREEN and shift if outside screen (right, bottom) frame

Function RestoreWinCoords(how, [name])
	variable how
	string name
	
	string theSizes
	variable left, top, right, bottom, owidth, oheight, offset
	variable oscrwpnt, oscrhpnt
	
	if (ParamIsDefault(name))
		name = WinName(0,(1+2+4+16+64),1)
	endif

	theSizes = GetUserData(name,"","ScreenSizer")

	if (strlen(theSizes)==0)
		return -1
	endif
	
	left = NumberByKey("left",theSizes,"=")
	if (NumType(left)==2)
		return -1
	endif
	top = NumberByKey("top",theSizes,"=")
	right = NumberByKey("right",theSizes,"=")
	bottom = NumberByKey("bottom",theSizes,"=")
	oscrwpnt = NumberByKey("width",theSizes,"=")
	oscrhpnt = NumberByKey("height",theSizes,"=")
	owidth = right - left
	oheight = bottom - top
	
	Initialize(0)
	
	NVAR scrwpnt = root:Packages:ScreenSizer:scrwpnt
	NVAR scrhpnt = root:Packages:ScreenSizer:scrhpnt

	switch(WinType(name))
		case 0:
			return -1
		case 13:
			return -1
		case 7:	// panels always restore as either -1 or 0 or 2 ONLY
			switch(how)
				case -1:
					break
				case 0:
				case 1:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					bottom = top + oheight
					break
				case 2:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					if (right>scrwpnt*100)
						offset = scrwpnt*100 - right
						left = (left-offset) < 0 ? 0 : (left-offset)
						right = left + owidth
					endif
					bottom = top + oheight
					if (bottom>scrhpnt*100)
						offset = scrhpnt*100 - bottom
						top = (top-offset) < 0 ? 0 : (top-offset)
						bottom = top + oheight
					endif
					break
			endswitch
			break
		default:
			switch(how)
				case -1:
					break
				case 0:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					bottom = top + oheight
					break
				case 1:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + (owidth/oscrwpnt)*scrwpnt
					bottom = top + (oheight/oscrhpnt)*scrhpnt
					break
				case 2:
					left = (left/oscrwpnt)*scrwpnt
					top = (top/oscrhpnt)*scrhpnt
					right = left + owidth
					if (right>scrwpnt*100)
						offset = scrwpnt*100 - right
						left = (left-offset) < 0 ? 0 : (left-offset)
						right = left + owidth
					endif
					bottom = top + oheight
					if (bottom>scrhpnt*100)
						offset = scrhpnt*100 - bottom
						top = (top-offset) < 0 ? 0 : (top-offset)
						bottom = top + oheight
					endif
					break
			endswitch
	endswitch
	MoveWindow/W=$name left, top, right, bottom
	return 0
end

Function SaveAllWindows()

	string theList = WinList("*",";","WIN:87"), theOne
	variable ic=0, nt = ItemsInList(theList)
	for(ic=nt;ic>=0;ic+=-1)
		theOne = StringFromList(ic,theList)
		SaveWinCoords(name=theOne)
	endfor
	return 0
end

Function RestoreAllWindows(how)
	variable how

	string theList = WinList("*",";","WIN:87"), theOne
	variable ic=0, nt=ItemsInList(theList)
	for(ic=nt-1;ic>=0;ic+=-1)
		theOne = StringFromList(ic,theList)
		RestoreWinCoords(how,name=theOne)
	endfor
	return 0
end

// Percentage of Screen Width left After Offset in piXels

Function pWaOx(offset)
	variable offset

	Initialize(0)
	
	NVAR scrleft = root:Packages:ScreenSizer:scrleft
	NVAR scrright = root:Packages:ScreenSizer:scrright

	NVAR scrwpxl = root:Packages:ScreenSizer:scrwpxl
	
	variable rtV
	
	 rtV = ((scrright - scrleft)*scrwpxl) - offset
	 
	 return (100*rtV/(rtV+offset))
	
end

// Percentage of Screen Height left After Offset in piXels

Function pHaOx(offset)
	variable offset

	Initialize(0)
	
	NVAR scrtop = root:Packages:ScreenSizer:scrtop
	NVAR scrbottom = root:Packages:ScreenSizer:scrbottom

	NVAR scrhpxl = root:Packages:ScreenSizer:scrhpxl
	
	variable rtV
	
	 rtV = ((scrbottom - scrtop)*scrhpxl) - offset
	 
	 return (100*rtV/(rtV+offset))
	
end

// Help Function

Function Help()

	DisplayHelpTopic "ScreenSizer"
	return 0
end
